#TCGA Merge 2.0
#v2: Modified to pull from an archive w/ file manifest and sample files.
#Rob Borkowski, robert.borkowski@utsouthwestern.edu, 19 Oct 2011.
#take TCGA data downloads and merge into one coherent file that preserves sample ID info. Will create a "merged.txt" file with all samples and probes in a flat file in r x c format. Use "rxc_to_gct.pl" to convert to GenePattern-Friendly format. Designed for Level 3 miRNA expression data, should work on any L3 exp. data.
#useage: call this script by pointing it to a .zip file containing the "file_manifest.txt" file from the TCGA archive (you will probably need to move this file into the folder) and the per-sample expression files in the archive.
#syntax: perl tcga_merge.pl

{
    use Archive::Extract;
    
    $ae = Archive::Extract->new( archive => $ARGV[0]);
    $ok = $ae->extract or die $ae->error;

    
    #open file manifest, pull out all data file names
    $file = "";
    open (DATA, "<file_manifest.txt") || die ;
    while (<DATA>) {
        $file .= $_;
    }
    close DATA;

    @files = split/\n/, $file; #each line = array element
    shift(@files);shift(@files);shift(@files); #chop off header and metadata files
    
    foreach $f (@files) {
        chomp ($f);
        @f_fields = split /\t/, $f;
        push(@file_list, $f_fields[6]);
    }
    $flist = @file_list; #number of files
    
    #open and process each file.
    
    @header = ("ID");
    
    
    for ($i = 0; $i <= $flist-1; $i ++){
        
        $expr = "";
        open (DATA, "<$file_list[$i]") || die ;
        while (<DATA>) {
            $expr .= $_;
        }
        close DATA;
    
        @first = split /\n/, $expr;
    
        shift(@first);
            @value = (); @barcode = (); @mir = (); @fields = ();
        foreach $record (@first) {
            chomp($record);
            @fields = split /\t/, $record;
            push(@mir, $fields[1]);
            push(@value, $fields[2]);
            push(@barcode, $fields[0]);
        }
        $exp_val[$i] = [@value];
        $mirnas[$i] = [@mir];
        push(@header, $barcode[0]);
        
        $length = @value;
    }
    #write this shit out to one file
    
    open (OUTFILE, ">$ARGV[1].txt");
    
    foreach $x (@header) {
        print OUTFILE "$x\t";
    }
    print OUTFILE "\n";
    
    
     for ($i = 0; $i <= $length; $i ++){
       print OUTFILE "$mirnas[0][$i]\t";
       
       for($j = 0; $j <= $flist; $j ++){
           print OUTFILE "$exp_val[$j][$i]\t";
       }
       
       print OUTFILE "\n";
    }
    
    close OUTFILE || die;
}